@@ -1,11 +1,19 @@
+from __future__ import annotations
+
 from collections import OrderedDict
 import dataclasses as dc
 from itertools import chain, zip_longest
+from typing import Iterable, Iterator, List, MutableMapping, Optional, Set
+
 import regex
-from typing import Set
 
 from swtor_settings_updater.color import Color
-from swtor_settings_updater.util import *
+from swtor_settings_updater.util.character_class import (
+    CP1252_PRINTABLE,
+    regex_character_class,
+)
+from swtor_settings_updater.util.swtor_case import swtor_lower
+
 
 CUSTOM_CHANNEL_IXS = range(22, 28 + 1)
 MAXIMUM_IX = 37
@@ -14,93 +22,105 @@ DEFAULT_COLOR = Color(238, 238, 0)
 
 
 @dc.dataclass
-class StandardChannels:
+class Channel:
+    name: str
+    ix: int
+    color: Color = dc.field(default_factory=lambda: DEFAULT_COLOR.copy())
+
+
+@dc.dataclass
+class StandardChannels(Iterable[Channel]):
     # Global Channels
-    trade: Color = dc.field(
+    trade: Channel = dc.field(
         default_factory=lambda: Channel("Trade", 7, (Color(179, 236, 255)))
     )
-    pvp: Color = dc.field(
+    pvp: Channel = dc.field(
         default_factory=lambda: Channel("PvP", 8, (Color(179, 236, 255)))
     )
-    general: Color = dc.field(
+    general: Channel = dc.field(
         default_factory=lambda: Channel("General", 6, (Color(179, 236, 255)))
     )
 
     # Player Channels
-    emote: Color = dc.field(
+    emote: Channel = dc.field(
         default_factory=lambda: Channel("Emote", 2, (Color(255, 128, 34)))
     )
-    yell: Color = dc.field(
+    yell: Channel = dc.field(
         default_factory=lambda: Channel("Yell", 1, (Color(255, 115, 151)))
     )
-    officer: Color = dc.field(
+    officer: Channel = dc.field(
         default_factory=lambda: Channel("Officer", 11, (Color(255, 0, 255)))
     )
-    guild: Color = dc.field(
+    guild: Channel = dc.field(
         default_factory=lambda: Channel("Guild", 10, (Color(130, 236, 137)))
     )
-    say: Color = dc.field(
+    say: Channel = dc.field(
         default_factory=lambda: Channel("Say", 0, (Color(179, 236, 255)))
     )
-    whisper: Color = dc.field(
+    whisper: Channel = dc.field(
         default_factory=lambda: Channel("Whisper", 3, (Color(165, 159, 243)))
     )
-    ops: Color = dc.field(
+    ops: Channel = dc.field(
         default_factory=lambda: Channel("Ops", 12, (Color(239, 188, 85)))
     )
-    ops_leader: Color = dc.field(
+    ops_leader: Channel = dc.field(
         default_factory=lambda: Channel("Ops Leader", 29, (Color(255, 84, 0)))
     )
-    group: Color = dc.field(
+    group: Channel = dc.field(
         default_factory=lambda: Channel("Group", 9, (Color(29, 140, 254)))
     )
-    ops_announcement: Color = dc.field(
+    ops_announcement: Channel = dc.field(
         default_factory=lambda: Channel("Ops Announcement", 33, (Color(160, 0, 0)))
     )
-    ops_officer: Color = dc.field(
+    ops_officer: Channel = dc.field(
         default_factory=lambda: Channel("Ops Officer", 13, (Color(49, 122, 60)))
     )
 
     # System Channels
-    combat_information: Color = dc.field(
+    combat_information: Channel = dc.field(
         default_factory=lambda: Channel("Combat Information", 37, (Color(255, 102, 0)))
     )
-    conversation: Color = dc.field(
+    conversation: Channel = dc.field(
         default_factory=lambda: Channel("Conversation", 19, (Color(238, 238, 0)))
     )
-    character_login: Color = dc.field(
+    character_login: Channel = dc.field(
         default_factory=lambda: Channel("Character Login", 20, (Color(238, 238, 0)))
     )
-    ops_information: Color = dc.field(
+    ops_information: Channel = dc.field(
         default_factory=lambda: Channel("Ops Information", 34, (Color(201, 46, 86)))
     )
-    system_feedback: Color = dc.field(
+    system_feedback: Channel = dc.field(
         default_factory=lambda: Channel("System Feedback", 18, (Color(238, 238, 0)))
     )
-    guild_information: Color = dc.field(
+    guild_information: Channel = dc.field(
         default_factory=lambda: Channel("Guild Information", 36, (Color(31, 171, 41)))
     )
-    group_information: Color = dc.field(
+    group_information: Channel = dc.field(
         default_factory=lambda: Channel("Group Information", 35, (Color(187, 79, 210)))
     )
 
     # Not shown in the UI. The default General panel and any panel you
     # create displays these, but the default Other panel does not.
-    error: Color = dc.field(
+    error: Channel = dc.field(
         default_factory=lambda: Channel("Error", 15, (Color(255, 0, 0)))
     )
-    server_admin: Color = dc.field(
+    server_admin: Channel = dc.field(
         default_factory=lambda: Channel("Server Admin", 17, (Color(255, 127, 127)))
     )
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator[Channel]:
         return (getattr(self, f.name) for f in dc.fields(self))
 
 
 class Chat:
     """Manage chat panels, custom channels and colors."""
 
-    def __init__(self):
+    standard_channels: StandardChannels
+    panels: OrderedDict[str, Panel]
+    custom_channel_ixs_available: List[int]
+    custom_channels: OrderedDict[str, CustomChannel]
+
+    def __init__(self) -> None:
         self.standard_channels = StandardChannels()
 
         self.panels = OrderedDict()
@@ -111,7 +131,7 @@ class Chat:
 
         self.custom_channels = OrderedDict()
 
-    def panel(self, name):
+    def panel(self, name: str) -> Panel:
         """Create a chat panel."""
         name_lower = swtor_lower(name)
 
@@ -123,7 +143,9 @@ class Chat:
 
         return t
 
-    def custom_channel(self, name, password=None, id=None):
+    def custom_channel(
+        self, name: str, password: Optional[str] = None, id: Optional[str] = None
+    ) -> CustomChannel:
         """Create a custom chat channel."""
         name_lower = swtor_lower(name)
 
@@ -139,14 +161,15 @@ class Chat:
 
         return cc
 
-    def apply(self, settings):
+    def apply(self, settings: MutableMapping[str, str]) -> None:
         """Apply the chat settings to a configuration object."""
         settings["ChatChannels"] = self.panels_setting()
         settings["Chat_Custom_Channels"] = self.custom_channels_setting()
         settings["ChatColors"] = self.colors_setting()
 
-    def panels_setting(self):
+    def panels_setting(self) -> str:
         """Compute the value for the panels setting (ChatChannels)."""
+        panels: Iterable[Panel]
         if self.panels:
             panels = self.panels.values()
         else:
@@ -167,7 +190,7 @@ class Chat:
             channels.append(f"{num}.{panel.name}.{panel.channel_bitmask()};")
         return "".join(channels)
 
-    def undisplayed_channel_ixs(self):
+    def undisplayed_channel_ixs(self) -> Set[int]:
         """Compute the indices of channels not displayed on any panel."""
         ixs = set()
 
@@ -182,7 +205,7 @@ class Chat:
 
         return ixs
 
-    def custom_channels_setting(self):
+    def custom_channels_setting(self) -> str:
         """Compute the value for the Chat_Custom_Channels setting."""
         custom_channels = []
         for num, cc in enumerate(self.custom_channels.values(), start=1):
@@ -190,7 +213,7 @@ class Chat:
             custom_channels.append(f"{cc.name};{password};{num};{cc.id}")
         return ";".join(custom_channels)
 
-    def colors_setting(self):
+    def colors_setting(self) -> str:
         """Compute the value for the ChatColors setting."""
         colors = [DEFAULT_COLOR.copy() for _ in range(MAXIMUM_IX + 1)]
         for c in chain(self.standard_channels, self.custom_channels.values()):
@@ -206,33 +229,26 @@ class Panel:
     # . and ; are separators.
     NAME_REGEX = regex.compile(f'[{regex_character_class(CP1252_PRINTABLE, ".;")}]+')
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         if not regex.fullmatch(Panel.NAME_REGEX, self.name):
             raise ValueError(f"Invalid name {self.name!r}")
 
-    def display(self, *channels):
+    def display(self, *channels: Channel) -> None:
         """Display the given channel(s) on the panel."""
         for c in channels:
             self.channel_ixs.add(c.ix)
 
-    def channel_bitmask(self):
+    def channel_bitmask(self) -> int:
         bitmask = 0
         for ix in self.channel_ixs:
             bitmask |= 1 << ix
         return bitmask
 
 
-@dc.dataclass
-class Channel:
-    name: str
-    ix: int
-    color: Color = dc.field(default_factory=lambda: DEFAULT_COLOR.copy())
-
-
 @dc.dataclass
 class CustomChannel(Channel):
-    password: str = None
-    id: str = None
+    password: Optional[str] = None
+    id: Optional[str] = None
 
     NAME_REGEX = regex.compile(r"[A-Za-z0-9_]+")
     # Spaces are not allowed, ; is a separator, and "&<> seem to be encoded by
@@ -242,14 +258,14 @@ class CustomChannel(Channel):
     )
     ID_REGEX = regex.compile(r"usr\.[a-z0-9_]+")
 
-    def __post_init__(self):
+    def __post_init__(self) -> None:
         if not regex.fullmatch(CustomChannel.NAME_REGEX, self.name):
             raise ValueError(f"Invalid name {self.name!r}")
 
         if self.password is not None and not regex.fullmatch(
             CustomChannel.PASSWORD_REGEX, self.password
         ):
-            raise ValueError(f"Invalid password {password!r}")
+            raise ValueError(f"Invalid password {self.password!r}")
 
         if self.id is not None and not regex.fullmatch(CustomChannel.ID_REGEX, self.id):
             raise ValueError(f"Invalid id {id!r}")
